"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.entitize = entitize;
exports.findAssociated = findAssociated;
exports.findById = findById;

var _lodash = require("lodash");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function entitize(json) {
  var entities = {};
  entities = reduceData(entities, json.data);
  entities = reduceData(entities, json.included);
  return { entities: entities };
}

function findAssociated(store, resourceType, id, assocation) {
  var chosenAssociation = (0, _lodash.get)(store, ["entities", resourceType, id, "relationships", assocation], {});

  if ((0, _lodash.isArray)(chosenAssociation)) {
    return (0, _lodash.map)(chosenAssociation, function (association) {
      return findById(store, association.type, association.id);
    });
  } else {
    return findById(store, chosenAssociation.type, chosenAssociation.id);
  }
}

function findById(store, resourceType, id) {
  return (0, _lodash.get)(store, ["entities", resourceType, id], {});
}

function reduceData(entities, data) {
  return (0, _lodash.merge)({}, entities, (0, _lodash.reduce)(data, function (result, item) {
    var id = item.id;
    var type = item.type;
    var relationships = item.relationships;
    var attributes = item.attributes;


    result = createMisingType(result, type);
    result = setRecord(result, type, id, attributes);
    result = setRelationships(result, type, id, relationships);

    return result;
  }, {}));
}

function createMisingType(entities, type) {
  if ((0, _lodash.has)(entities, type)) {
    return entities;
  } else {
    return (0, _lodash.merge)({}, entities, entities[type] = {});
  }
}

function setRecord(result, type, id, attributes) {
  var newRecord = _defineProperty({}, type, _defineProperty({}, id, (0, _lodash.merge)({}, { id: id }, attributes)));

  return (0, _lodash.merge)({}, result, newRecord);
}

function setRelationships(entities, type, id, relationships) {
  var newRelationships = (0, _lodash.reduce)(relationships, function (result, relationship, key) {
    return _defineProperty({}, key, relationship.data);
  }, {});

  var data = _defineProperty({}, type, _defineProperty({}, id, {
    relationships: newRelationships
  }));

  return (0, _lodash.merge)({}, entities, data);
}